/*	______________________________________________________________________

	geometry.cpp  Version 1.02
	(C)2010. Kruchinin Aleksandr. http://vidikon.com
	 	
	______________________________________________________________________
*/

#include "geometry.h"

//       
F_LINE MakeLine(double x1,double y1,double x2,double y2)
{
	F_LINE f;
	f.k1=3;
		//    i3  i4
	f.b=1;
	if (absf(x1-x2)<absf(y1-y2)) f.b=2;
	if (f.b==1)
	{
		//y=f(x);
		f.b1=(double)(y2-y1)/(x2-x1);
		f.b2=(double)y1-(double)x1*f.b1;
	}
	if (f.b==2)
	{
		//x=f(y);
		f.b1=(double)(x2-x1)/(y2-y1);
		f.b2=(double)x1-(double)y1*f.b1;
	}
	return f;
}


F_LINE MakeLine(C_POINT p1,C_POINT p2)
{
	return MakeLine(p1.x,p1.y,p2.x,p2.y);
}
F_LINE MakeLine(C_POINTd p1,C_POINTd p2)
{
	return MakeLine(p1.x,p1.y,p2.x,p2.y);
}
/*
    line    
p1  p2     p,   
 p1  p2   p1
*/
C_POINTd FindPointContinue(C_POINTd p1,C_POINTd p2,F_LINE line,double size)
{
	C_POINTd p;
	p.x=-1;p.y=-1;

	C_POINTd p_t;
	//    x     y
	if (line.b==1)
	{
		//y=f(x);
		p_t.x=(double)size/sqrt((double)pow(line.b1,2)+1);
		p_t.y=absf((double)line.b1*p_t.x);
	}
	if (line.b==2)
	{
		//x=f(y);
		p_t.y=(double)size/sqrt((double)pow(line.b1,2)+1);
		p_t.x=absf((double)line.b1*p_t.y);
	}
	if (p2.x>p1.x) p.x=p1.x-p_t.x;
	else p.x=p1.x+p_t.x;

	if (p2.y>p1.y) p.y=p1.y-p_t.y;
	else p.y=p1.y+p_t.y;

	return p;
}

// -
C_POINTd FindPointContinue_(C_POINTd p1,C_POINTd p2,F_LINE line,double size)
{
	C_POINTd p;
	p.x=-1;p.y=-1;

	C_POINTd p_t;
	//    x     y
	if (line.b==1)
	{
		//y=f(x);
		p_t.x=(double)size/sqrt((double)pow(line.b1,2)+1);
		p_t.y=absf((double)line.b1*p_t.x);
	}
	if (line.b==2)
	{
		//x=f(y);
		p_t.y=(double)size/sqrt((double)pow(line.b1,2)+1);
		p_t.x=absf((double)line.b1*p_t.y);
	}
	if (p2.x>p1.x) p.x=p1.x+p_t.x;
	else p.x=p1.x-p_t.x;

	if (p2.y>p1.y) p.y=p1.y+p_t.y;
	else p.y=p1.y-p_t.y;

	return p;
}

//  
//1 -       
//0 - 
int Intersection(F_LINE f1,F_LINE f2,double &x,double &y)
{
	if (f1.b==1 && f2.b==1)
	{
		if (absf(f1.b1-f2.b1)<0.01) return 1;
		x=(double)(f2.b2-f1.b2)/(f1.b1-f2.b1);
		y=f1.b1*x+f1.b2;
	}
	if (f1.b==2 && f2.b==2)
	{
		if (absf(f1.b1-f2.b1)<0.01) return 1;
		y=(double)(f2.b2-f1.b2)/(f1.b1-f2.b1);
		x=f1.b1*y+f1.b2;
	}
	if (f1.b==1 && f2.b==2)
	{
		if (absf(1-f1.b1*f2.b1)<0.01) return 1;
		y=(double)(f1.b1*f2.b2+f1.b2)/(1-f1.b1*f2.b1);
		x=f2.b1*y+f2.b2;
	}
	if (f1.b==2 && f2.b==1)
	{
		if (absf(1-f1.b1*f2.b1)<0.01) return 1;
		y=(double)(f2.b1*f1.b2+f2.b2)/(1-f1.b1*f2.b1);
		x=f1.b1*y+f1.b2;
	}
	return 0;
}


double LengthLine(double x1,double y1,double x2,double y2)
{
	return (double)sqrt(pow(x2-x1,2)+pow(y2-y1,2));
}


inline float MakePolarR(float x,float y)
{
	return sqrt(pow(x,2)+pow(y,2));
}

inline float MakePolarF(float x,float y)
{
	float f=0.0;

	if (x>0 && y>=0) f=atan((double)(y)/(x));
	if (x<=0 && y>0) f=PI/2+atan(-(double)(x)/(y));
	if (x<0 && y<=0) f=PI+atan((double)(y)/(x));
	if (x>=0 && y<0) f=3*PI/2+atan(-(double)(x)/(y));

	return f;
}

bool DirectPoint(C_POINT*p,int all,unsigned char flags)
{
	double f[100];
	int i;
	int i1;
	for(i=0;i<all;i++)
	{
		if (i<all-1)
			f[i]=MakePolarF(p[i+1].x-p[i].x,p[i+1].y-p[i].y);
		else f[i]=MakePolarF(p[0].x-p[i].x,p[0].y-p[i].y);
	}
	if ((DIRECT_HOUR_ARROW&flags)==DIRECT_HOUR_ARROW) 
	{
		for(i=0;i<all;i++)
		{
			i1=i+1;
			if (i==all-1) i1=0;		
			if (f[i1]<f[i] && f[i]-f[i1]<=PI) continue;
			if (f[i1]>f[i] && f[i1]-f[i]>PI) continue;
			return false;
		}
	}
	if ((DIRECT_HOUR_ARROW_A&flags)==DIRECT_HOUR_ARROW_A) 
	{
		for(i=0;i<all;i++)
		{
			i1=i+1;
			if (i==all-1) i1=0;		
			if (f[i1]>f[i] && f[i1]-f[i]<=PI) continue;
			if (f[i1]<f[i] && f[i]-f[i1]>PI) continue;
			return false;
		}
	}
	return true;
}

C_POINTd PointID(C_POINT p) { C_POINTd d;d.x=p.x;d.y=p.y;return d;}
C_POINT PointDI(C_POINTd p) { C_POINT d;d.x=round(p.x);d.y=round(p.y);return d;}


double RToLine(F_LINE f,double x,double y)
{
	double a,a1;


	if (f.b==1)
	{
		if (absf((double)f.b1)<0.001) return abs(y-f.b2);
		a=(double)f.b1*x+f.b2;
		a1=(double)(y-f.b2)/f.b1;
		a=absf(a-y);
		a1=absf(a1-x);
	}
	else
	{
		if (absf((double)f.b1)<0.001) return abs(x-f.b2);
		a=(double)f.b1*y+f.b2;
		a1=(double)(x-f.b2)/f.b1;
		a=absf(a-x);
		a1=absf(a1-y);
	}
	//return (double)a*a1/sqrt(a*a+a1*a1);
	return (double)a1*sin(atan((double)a/a1));

}

double RToLine(F_LINE f,C_POINTd p)
{
	return RToLine(f,p.x,p.y);
}




int round(double param)
{
	int p=param;
	if (param-0.5<p) return p;
	else return p+1;
}

double absf(double a)
{
	if (a<0) return -a;
	return a;
}


C_POINTd Average(C_POINTd p1,C_POINTd p2)
{
	C_POINTd p;
	p.x=(p1.x+p2.x)/2;
	p.y=(p1.y+p2.y)/2;

	return p;
}

//  
F_LINE MakeParallelLine(F_LINE line,C_POINTd point)
{
	F_LINE line1;

	line1.b=line.b;
	line1.b1=line.b1;
	line1.k1=line.k1;
	if (line.b==1)
	{
		line1.b2=(double)point.y-line1.b1*point.x;
	}
	if (line.b==2)
	{
		line1.b2=(double)point.x-line1.b1*point.y;
	}

	return line1;
}

double FoundInclination(F_LINE line,F_LINE line1)
{
	double alfa=0;
	
	double x1,y1;

	if (line.b==1) { x1=1; y1=x1*line.b1;}
	else { y1=1; x1=y1*line.b1;}

	double f1=MakePolarF(x1,y1);

	if (line1.b==1) { x1=1; y1=x1*line1.b1;}
	else { y1=1; x1=y1*line1.b1;}

	double f2=MakePolarF(x1,y1);
	
	//if (f2>=3*PI/2) f2=2*PI-f2;
	//if (f1>=3*PI/2) f1=2*PI-f1;
	if (f2>=PI) f2=f2-PI;
	if (f1>=PI) f1=f1-PI;

	int i=1;// line  ( )  line1
	if ((f2>f1 && f2-f1<PI/2) || (f2<f1 && f1-f2>PI/2)) i=-1;

	alfa=absf(f1-f2);
	if (alfa>PI/2) alfa=PI-alfa;// 2.6

	return (double)alfa*i;
}

F_LINE TurnLine(F_LINE f,double alfa,C_POINTd point)
{
	F_LINE a;
	double x,y;
	double f1,r;
	C_POINTd pp;
	
	if (f.b==1)
	{
		x=0;
		y=(double)f.b1*x+f.b2;
		r=MakePolarR(1,(double)f.b1+f.b2-y);
		f1=MakePolarF(1,(double)f.b1+f.b2-y);
		if (f1>=PI) f1=f1-PI;
		f1=f1-alfa;		
	}
	if (f.b==2)
	{
		y=0;
		x=(double)f.b1*y+f.b2;
		r=MakePolarR((double)f.b1+f.b2-x,1);
		f1=MakePolarF((double)f.b1+f.b2-x,1);
		if (f1>=PI) f1=f1-PI;
		f1=f1-alfa;		
	}

	pp.x=x;
	pp.y=y;

	a=MakeLineFromPolar(pp,f1);
	a=MakeParallelLine(a,point);

	return a;
}


F_LINE MakeLineFromPolar(C_POINTd p1,double f)
{
	F_LINE a;

	if (f>=PI) f=f-PI;
	if (f<0) f=PI+f;

	if (f>=0 && f<(double)PI/4) { a.b=1; a.b1=tan(f);}
	if (f>=(double)PI/4 && f<(double)PI/2) { a.b=2; a.b1=tan(PI/2-f);}
	if (f>=(double)PI/2 && f<(double)3*PI/4) { a.b=2; a.b1=-tan(f-PI/2);}
	if (f>=(double)3*PI/4 && f<(double)PI) { a.b=1; a.b1=-tan(PI-f);}

	if (a.b==1) a.b2=p1.y-p1.x*a.b1;	
	if (a.b==2) a.b2=p1.x-p1.y*a.b1;	

	return a;
}


//   
//     array     all_elem
F_LINE Approximation(int *array,int all_elem)
{
	F_LINE f;
	f.b=-1;
	int i;
	long sx,sxx,sy,sxy;

	sx=0;sxx=0;sy=0;sxy=0;
	for(i=0;i<all_elem;i++)
	{
		sx+=i;
		sxx+=i*i;
		sy+=array[i];
		sxy+=i*array[i];
	}

	long delta=all_elem*sxx-sx*sx;
	f.b=1;
	f.b2=(double)(sxx*sy-sx*sxy)/delta;
	f.b1=(double)(all_elem*sxy-sx*sy)/delta;
	return f;
}
