/*	______________________________________________________________________

	histogram.cpp  Version 1.00
	(C)2010. Kruchinin Aleksandr. http://vidikon.com
	Histogram -   
	______________________________________________________________________
*/

#include "histogram.h"


//   
HISTO_1D MakeHistogramIMG(IMG Img,RECT Rect,int AllElem)
{
	HISTO_1D h;
	h.AllElem=-1;

	if (Img.format!=1 || AllElem<1) return h;

	int i,j,k;
	

	h=CreateHistogram(AllElem);

	for(i=Rect.top;i<=Rect.bottom;i++)
		for(j=Rect.left;j<=Rect.right;j++)
		{
			k=(int)Img.bytes[j+i*Img.Width_1]*AllElem/256;			
			if (k>=h.AllElem) k=h.AllElem-1;
			h.data[k]++;
		}
	h.pixels=0;
	for(i=0;i<h.AllElem;i++)
		h.pixels+=h.data[i];

	return h;
}


HISTO_1D CreateHistogram(int AllElem)
{
	HISTO_1D h;
	h.AllElem=-1;

	if (AllElem<1) return h;

	h.data=new int[AllElem];
	h.AllElem=AllElem;

	memset((void*)h.data,0,h.AllElem*sizeof(int));

	return h;
}

void DeleteHistogram(HISTO_1D h)
{
	if (h.AllElem>-1)
		if (h.data!=NULL) delete h.data;
}

int MaxHistogramElem(HISTO_1D h)
{
	if (h.AllElem<1 || h.data==NULL) return -1;

	int i,j=0;

	for(i=1;i<h.AllElem;i++)
		if (h.data[i]>h.data[j]) j=i;

	return j;
}

/*
0 - 
1 -   
2 -   
*/
int DrawHistogram(HISTO_1D h,IMG Img)
{
	if (Img.format!=0) return 1;
	if (h.AllElem<1 || h.data==NULL) return 2;

	C_POINT p1,p2;
	
	ClearImage(Img);
	p1.x=0;p1.y=0;
	p2.x=Img.Width-1;p2.y=0;
	DrawLine(Img,p1,p2,128,128,128);

	int i;
	RECT Rect;
	int j=MaxHistogramElem(h);
	double j1;

	for(i=0;i<h.AllElem;i++)
	{
		j1=(double)(Img.Height-3)*h.data[i]/h.data[j];
		Rect.left=i*Img.Width/h.AllElem;
		Rect.right=(i+1)*Img.Width/h.AllElem;
		Rect.top=1;
		Rect.bottom=j1;
		DrawRect(Img,Rect,255,255,255);
	}

	return 0;
}

//      
double CompareHistogram(HISTO_1D h1,HISTO_1D h2)
{
	double kpk;

	double m2,m1;
	m1=M_e(h1.AllElem,h1.data);
	m2=M_e(h2.AllElem,h2.data);

	kpk=KPK_e(h1.AllElem,h1.data,h2.data,m1,m2,D_e(h1.AllElem,h1.data,m1),D_e(h2.AllElem,h2.data,m2));

	return kpk;
}

//    ,  %   
int CompareHistogram1(HISTO_1D h1,HISTO_1D h2,double threshold)
{
	int i,j=0;
	int j1=0;
	for(i=0;i<h1.AllElem;i++)
		if (h1.data[i]>(double)h1.pixels*threshold) j++;
	for(i=0;i<h2.AllElem;i++)
		if (h2.data[i]>(double)h2.pixels*threshold) j1++;
	return absf(j1-j);	
}


double CompareHist(HISTO_1D h1,HISTO_1D h2,DWORD flags)
{
	double d=0;

	if ((HIST_COMP_CHISQR&flags)==HIST_COMP_CHISQR) 
	for(int i=0;i<h1.AllElem;i++)
	{
		if (h1.data[i]==0 && h2.data[i]==0) continue;
		d+=(double)pow((double)h1.data[i]-h2.data[i],2)/(h1.data[i]+h2.data[i]);
	}

	return d;
}